@@ -1,8 +1,8 @@
 /**************************************************************************
  *   browser.c  --  This file is part of GNU nano.                        *
  *                                                                        *
- *   Copyright (C) 2001-2011, 2013-2025 Free Software Foundation, Inc.    *
- *   Copyright (C) 2015-2016, 2020, 2022 Benno Schulenberg                *
+ *   Copyright (C) 2001-2011, 2013-2026 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2015, 2016, 2020, 2022, 2025 Benno Schulenberg         *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -15,7 +15,7 @@
  *   See the GNU General Public License for more details.                 *
  *                                                                        *
  *   You should have received a copy of the GNU General Public License    *
- *   along with this program.  If not, see http://www.gnu.org/licenses/.  *
+ *   along with this program.  If not, see https://gnu.org/licenses/.     *
  *                                                                        *
  **************************************************************************/
 
@@ -353,13 +353,13 @@ void research_filename(bool forwards)
 	}
 }
 
-/* Select the first file in the list -- called by ^W^Y. */
+/* Select the first file in the list -- called directly by ^W^Y. */
 void to_first_file(void)
 {
 	selected = 0;
 }
 
-/* Select the last file in the list -- called by ^W^V. */
+/* Select the last file in the list -- called directly by ^W^V. */
 void to_last_file(void)
 {
 	selected = list_length - 1;
@@ -421,6 +421,10 @@ char *browse(char *path)
 		dir = NULL;
 	}
 
+#ifndef NANO_TINY
+	resized_for_browser = FALSE;
+#endif
+
 	/* If something was selected before, reselect it;
 	 * otherwise, just select the first item (..). */
 	if (present_name != NULL) {
@@ -486,11 +490,12 @@ char *browse(char *path)
 
 		function = interpret(kbinput);
 
-		if (function == do_help || function == full_refresh) {
-			function();
-#ifndef NANO_TINY
-			/* Simulate a terminal resize to force a directory reread,
-			 * or because the terminal dimensions might have changed. */
+		if (function == do_help)
+			do_help();
+		else if (function == full_refresh) {
+#ifdef NANO_TINY
+			full_refresh();
+#else
 			kbinput = THE_WINDOW_RESIZED;
 		} else if (function == do_toggle && get_shortcut(kbinput)->toggle == NO_HELP) {
 			TOGGLE(NO_HELP);
@@ -548,17 +553,15 @@ char *browse(char *path)
 								list_length - piles;
 			else
 				selected += usable_rows * piles;
-		} else if (function == to_first_file) {
-			selected = 0;
-		} else if (function == to_last_file) {
-			selected = list_length - 1;
+		} else if (function == to_first_file || function == to_last_file) {
+			function();
 		} else if (function == goto_dir) {
 			/* Ask for the directory to go to. */
 			if (do_prompt(MGOTODIR, "", NULL,
 							/* TRANSLATORS: This is a prompt. */
 							browser_refresh, _("Go To Directory")) < 0) {
 				statusbar(_("Cancelled"));
-				continue;
+				goto testresize;
 			}
 
 			path = free_and_assign(path, real_dir_from_tilde(answer));
@@ -575,7 +578,7 @@ char *browse(char *path)
 				 * the option --operatingdir, not of --restricted. */
 				statusline(ALERT, _("Can't go outside of %s"), operating_dir);
 				path = mallocstrcpy(path, present_path);
-				continue;
+				goto testresize;
 			}
 #endif
 			/* Snip any trailing slashes, so the name can be compared. */
@@ -646,9 +649,10 @@ char *browse(char *path)
 		} else
 			unbound_key(kbinput);
 
+  testresize:
 #ifndef NANO_TINY
 		/* If the terminal resized (or might have), refresh the file list. */
-		if (kbinput == THE_WINDOW_RESIZED) {
+		if (kbinput == THE_WINDOW_RESIZED || resized_for_browser) {
 			/* Remember the selected file, to be able to reselect it. */
 			present_name = copy_of(filelist[selected]);
 			goto read_directory_contents;
