@@ -1,7 +1,7 @@
 /**************************************************************************
  *   history.c  --  This file is part of GNU nano.                        *
  *                                                                        *
- *   Copyright (C) 2003-2011, 2013-2025 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2003-2011, 2013-2026 Free Software Foundation, Inc.    *
  *   Copyright (C) 2016, 2017, 2019, 2025 Benno Schulenberg               *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -537,10 +537,8 @@ void update_poshistory(void)
 	poshiststruct *previous = NULL;
 	poshiststruct *item;
 
-	if (fullpath == NULL || openfile->filename[0] == '\0') {
-		free(fullpath);
+	if (fullpath == NULL)
 		return;
-	}
 
 	reload_positions_if_needed();
 
@@ -597,6 +595,6 @@ void restore_cursor_position_if_any(void)
 	if (item && item->anchors)
 		restore_anchors(item->anchors);
 	if (item)
-		goto_line_and_column(item->linenumber, item->columnnumber, FALSE, FALSE);
+		goto_line_and_column(item->linenumber, item->columnnumber, TRUE);
 }
 #endif /* ENABLE_HISTORIES */
